<?php
/**
* @package RSForm!Pro
* @copyright (C) 2007-2019 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

class plgSystemRsfpvtiger extends JPlugin
{
	protected $autoloadLanguage = true;

	public function onRsformFormSave($form)
	{
		$app                = JFactory::getApplication();
		$data               = $app->input->post->get('vt', array(), 'array');
		$data['vt_fields']  = $app->input->post->get('vt_fields', array(), 'array');
		$data['form_id']    = $form->FormId;

		$row = $this->getTable();
		if (!$row)
		{
			return false;
		}

		if (!$row->save($data))
		{
			$app->enqueueMessage($row->getError(), 'error');
		}
	}
	
	public function onRsformBackendFormCopy($args)
	{
		$formId     = $args['formId'];
		$newFormId  = $args['newFormId'];

		if ($row = $this->getTable())
		{
			if ($row->load($formId))
			{
				if (!$row->bind(array('form_id' => $newFormId)))
				{
					return false;
				}

				$row->check();

				return $row->store();
			}
		}
	}

	protected function getTable()
	{
		return JTable::getInstance('RSForm_vTiger', 'Table');
	}
	
	public function onRsformBackendAfterShowFormEditTabs()
	{
		$app	= JFactory::getApplication();
		$formId = $app->input->getInt('formId');
		
		$row = $this->getTable();
		if (!$row)
		{
			return false;
		}
		$row->load($formId);
		
		// Unserialize fields if this value is populated.
		if ($row->vt_fields)
		{
			$row->vt_fields = unserialize($row->vt_fields);
		}
		
		// Make sure it's an array.
		if (!is_array($row->vt_fields)) {
			$row->vt_fields = array();
		}
		
		$lists['published'] = RSFormProHelper::renderHTML('select.booleanlist', 'vt[vt_published]', '', $row->vt_published);
		$lists['debug'] 	= RSFormProHelper::renderHTML('select.booleanlist', 'vt[vt_debug]', '', $row->vt_debug);
		$fields				= false;
		
		if ($row->vt_published)
		{
			// Show errors if plugin is not configured correctly.
			if (!$row->vt_accesskey || !$row->vt_username || !$row->vt_hostname)
			{
				// No access key
				if (!$row->vt_accesskey)
				{
					$app->enqueueMessage(JText::_('RSFP_VT_PLEASE_SUPPLY_YOUR_ACCESS_KEY'), 'warning');
				}
				// No username
				if (!$row->vt_username)
				{
					$app->enqueueMessage(JText::_('RSFP_VT_PLEASE_SUPPLY_YOUR_USERNAME'), 'warning');
				}
				// No host setup
				if (!$row->vt_hostname)
				{
					$app->enqueueMessage(JText::_('RSFP_VT_PLEASE_SUPPLY_YOUR_HOSTNAME'), 'warning');
				}
				
				// Info
				$app->enqueueMessage(JText::_('RSFP_VT_SURPRESS_ERRORS_DISABLE_INTEGRATION'), 'warning');
			}
			else
			{
				// Connect to Vtiger and grab fields.
				try
				{
					list($sessionName, $userId) = $this->login($row->vt_username, $row->vt_accesskey, $row->vt_hostname);
					
					$response = $this->webservice($row->vt_hostname, $row->vt_username, array(
						'operation' 	=> 'describe',
						'sessionName' 	=> $sessionName,
						'elementType' 	=> 'Leads'
					));
					
					$fields = $response['result']['fields'];
				}
				catch (Exception $e)
				{
					JFactory::getApplication()->enqueueMessage(JText::sprintf('RSFP_VT_ERROR_MESSAGE', $e->getMessage()), 'error');
				}
			}
		}
		
		?>
		<div id="vtigerdiv">
			<h3 class="rsfp-legend"><?php echo JText::_('RSFP_VTIGER_INTEGRATION'); ?></h3>
			<div class="alert alert-info"><?php echo JText::_('RSFP_VTIGER_DESC'); ?></div>
			<table class="table table-bordered">
				<tr>
					<td nowrap><?php echo JText::_('RSFP_VT_USE_INTEGRATION'); ?></td>
					<td><?php echo $lists['published']; ?></td>
				</tr>
				<tr>
					<td nowrap><?php echo JText::_('RSFP_VT_SHOW_DEBUG_MESSAGES'); ?></td>
					<td><?php echo $lists['debug']; ?></td>
				</tr>
			</table>

			<h3 class="rsfp-legend"><?php echo JText::_('RSFP_VT_AUTHENTICATION'); ?></h3>
			<table class="table table-bordered">
				<tr>
					<td nowrap><?php echo JText::_('RSFP_VT_ACCESSKEY'); ?></td>
					<td>
						<input name="vt[vt_accesskey]" id="vt_accesskey" value="<?php echo RSFormProHelper::htmlEscape($row->vt_accesskey); ?>" class="rs_inp rs_80" />
					</td>
				</tr>
				<tr>
					<td nowrap><?php echo JText::_('RSFP_VT_USERNAME'); ?></td>
					<td>
						<input name="vt[vt_username]" id="vt_username" value="<?php echo RSFormProHelper::htmlEscape($row->vt_username); ?>" class="rs_inp rs_80" />
					</td>
				</tr>
				<tr>
					<td nowrap><?php echo JText::_('RSFP_VT_HOSTNAME'); ?></td>
					<td>
						<input name="vt[vt_hostname]" id="vt_hostname" value="<?php echo RSFormProHelper::htmlEscape($row->vt_hostname); ?>" class="rs_inp rs_80" />
					</td>
				</tr>
			</table>

			<h3 class="rsfp-legend"><?php echo JText::_('RSFP_VT_FIELDS'); ?></h3>
			<table class="table table-bordered">
				<?php
				if ($fields)
				{
					foreach ($fields as $f => $field)
					{
						if (!$field['editable'] || $field['type']['name'] == 'owner' || $field['type']['name'] == 'reference')
						{
							continue;
						}
						?>
						<tr>
							<td>
								<?php echo RSFormProHelper::htmlEscape($field['label']); ?> <?php if ($field['mandatory']) { ?><b style="color: red;">(*)</b><?php } ?>
							</td>
							<td>
								<input name="vt_fields[<?php echo RSFormProHelper::htmlEscape($field['name']); ?>]" id="vt_fields<?php echo RSFormProHelper::htmlEscape($field['name']); ?>" value="<?php echo RSFormProHelper::htmlEscape(isset($row->vt_fields[$field['name']]) ? $row->vt_fields[$field['name']] : ''); ?>" type="text" class="rs_inp rs_80"  data-delimiter=" " data-placeholders="display" />
							</td>
							<td>
								<?php
								echo RSFormProHelper::htmlEscape($field['type']['name']);
								if ($field['type']['name'] == 'picklist' || $field['type']['name'] == 'multipicklist')
								{
									$values = array();
									foreach ($field['type']['picklistValues'] as $value)
									{
										$values[] = RSFormProHelper::htmlEscape($value['value']);
									}
									if ($values)
									{
									?>
										<button type="button" class="btn btn-secondary btn-small btn-sm" onclick="jQuery('#vt_possible_values_<?php echo $f; ?>').toggle();"><?php echo JText::_('RSFP_VT_POSSIBLE_VALUES'); ?></button>
										<div id="vt_possible_values_<?php echo $f; ?>" style="display: none;">
											<p><?php echo implode('<br />', $values); ?></p>
										</div>
									<?php
									}
								}
								?>
							</td>
						</tr>
					<?php
					}
				}
				else
				{
					?>
					<tr>
						<td colspan="3">
							<p><?php echo JText::_('RSFP_VT_NOT_CONFIGURED'); ?></p>
							<ol>
								<li><?php echo JText::_('RSFP_VT_NOT_CONFIGURE_PROVIDE_ACCESS_KEY'); ?></li>
								<li><?php echo JText::_('RSFP_VT_NOT_CONFIGURE_PROVIDE_USERNAME'); ?></li>
								<li><?php echo JText::_('RSFP_VT_NOT_CONFIGURE_PROVIDE_HOST_URL'); ?></li>
							</ol>
							<p><?php echo JText::_('RSFP_VT_ONCE_CONFIGURED'); ?></p>
						</td>
					</tr>
				<?php
				}
				?>
			</table>
		</div>
<?php
	}
	
	protected function login($username, $accessKey, $hostname) {
		$response = $this->webservice($hostname, $username, array(
			'operation' => 'getchallenge',
			'username'  => $username
		));
		
		$token = $response['result']['token'];
		
		$response = $this->webservice($hostname, $username, array(
			'operation' => 'login',
			'username'  => $username,
			'accessKey' => md5($token . $accessKey),
		), true);
		
		return array($response['result']['sessionName'], $response['result']['userId']);
	}
	
	public function onRsformBackendAfterShowFormEditTabsTab()
	{
		echo '<li><a href="javascript: void(0);" id="vtiger"><span class="rsficon rsficon-paw"></span><span class="inner-text">'.JText::_('RSFP_VTIGER_INTEGRATION').'</span></a></li>';
	}
	
	public function onRsformFrontendAfterFormProcess($args)
	{
		$db				= JFactory::getDbo();
		$formId 		= (int) $args['formId'];
		$SubmissionId 	= (int) $args['SubmissionId'];

		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__rsform_vtiger'))
			->where($db->qn('form_id') . ' = ' . $db->q($formId))
			->where($db->qn('vt_published') . ' = ' . $db->q(1));

		$db->setQuery($query);
		if ($row = $db->loadObject()) {
			// Get replacements
			list($replace, $with) = RSFormProHelper::getReplacements($SubmissionId);
			
			// Add some of our own
			$replace[] = '\n';
			$with[]	   = "\n";

			// Grab configured fields
			$fields = array();
			if ($row->vt_fields) {
				$fields = unserialize($row->vt_fields);
			}
			
			if (!is_array($fields)) {
				$fields = array();
			}
			
			// Prepare the element ("Leads" in our case) properties.
			$element = array();
            foreach ($fields as $field => $value) {
				$element[$field] = str_replace($replace, $with, $value);
            }
			
			try {
				// Connect to Vtiger and login.
				list($sessionName, $userId) = $this->login($row->vt_username, $row->vt_accesskey, $row->vt_hostname);
				
				// Assign the Lead's user ID.
				$element['assigned_user_id'] =  $userId;
				
				// Create the Lead.
				$this->webservice($row->vt_hostname, $row->vt_username, array(
					'operation' 	=> 'create',
					'sessionName' 	=> $sessionName,
					'elementType' 	=> 'Leads',
					'element' 		=> json_encode($element)
				), true);
				
				// Logout
				$this->webservice($row->vt_hostname, $row->vt_username, array(
					'operation' => 'logout',
					'sessionName' => $sessionName,
				), true);
			} catch (Exception $e) {
            	if (!empty($row->vt_debug))
				{
					JFactory::getApplication()->enqueueMessage(JText::sprintf('RSFP_VT_ERROR_MESSAGE', $e->getMessage()), 'error');
				}
			}
		}
	}
    
    protected function webservice($hostname, $username, $data = null, $post = false) {
		// Create URL
		$url = rtrim($hostname, ' /').'/webservice.php';
		
		$http = JHttpFactory::getHttp();
	    $http->setOption('userAgent', "RSForm! Pro/Vtiger");
		
		if ($post)
		{
			$response = $http->post($url, $data);
		}
		else
		{
			$response = $http->get($url.'?'.http_build_query($data));
		}
		
		if ($response->code != 200)
		{
			throw new Exception(JText::sprintf('RSFP_VT_HTTP_CODE_RECEIVED', $response->code));
		}
		
        $json = $response->body;
		// Attempt to clean JSON response of extra characters.
		if (preg_match('/{.*}/', $json, $match))
		{
			$json = $match[0];
		}
		
		$data = json_decode($json, true);
		
		if (!$data)
		{
			throw new Exception(JText::_('RSFP_VT_COULD_NOT_PARSE_JSON_DATA'));
		}
		
		if (!$data['success'])
		{
			throw new Exception($data['error']['message']);
		}
		
		return $data;
    }
	
	public function onRsformFormDelete($formId)
	{
		if ($row = $this->getTable())
		{
			$row->delete($formId);
		}
	}
	
	public function onRsformFormBackup($form, $xml, $fields)
	{
		if ($row = $this->getTable())
		{
			if ($row->load($form->FormId))
			{
				$row->check();

				$data = $row->getProperties();
				unset($data['form_id']);

				$xml->add('vtiger');
				foreach ($data as $property => $value)
				{
					$xml->add($property, $value);
				}
				$xml->add('/vtiger');
			}
		}
	}
	
	public function onRsformFormRestore($form, $xml, $fields)
	{
		if (isset($xml->vtiger))
		{
			$data = array(
				'form_id' => $form->FormId
			);
			
			foreach ($xml->vtiger->children() as $property => $value)
			{
				$data[$property] = (string) $value;
			}
			
			$row = $this->getTable();
			$row->save($data);
		}
	}
	
	public function onRsformBackendFormRestoreTruncate()
	{
		JFactory::getDbo()->truncateTable('#__rsform_vtiger');
	}
}